@@ -1,9 +1,9 @@
 #! /usr/bin/env/python
 # -*- coding: utf-8 -*-
-# cython: language_level=3
+# cython: language_level=3, always_allow_keywords=True
 
-## Copyright 2005-2013 by LivingLogic AG, Bayreuth/Germany.
-## Copyright 2005-2013 by Walter Dörwald
+## Copyright 2005-2014 by LivingLogic AG, Bayreuth/Germany
+## Copyright 2005-2014 by Walter Dörwald
 ##
 ## All Rights Reserved
 ##
@@ -12,7 +12,7 @@
 
 import sys, re
 
-from ll import misc
+from ll import misc, ul4c
 
 import pytest
 
@@ -94,16 +94,14 @@ def test_item():
 	assert misc.item(e, 0) == 0
 	assert misc.item(e, 0) == 1
 	assert misc.item(e, -1) == 9
-	with pytest.raises(IndexError):
-		misc.item(e, -1)
+	assert misc.item(e, -1) is None
 	assert misc.item(e, -1, 42) == 42
 
 	e = iter(range(10))
 	assert misc.item(e, 4) == 4
 
 	e = iter(range(10))
-	with pytest.raises(IndexError):
-		misc.item(e, 10)
+	assert misc.item(e, 10) is None
 
 	e = iter(range(10))
 	assert misc.item(e, 10, 42) == 42
@@ -115,8 +113,7 @@ def test_item():
 	assert misc.item(e, -10) == 0
 
 	e = iter(range(10))
-	with pytest.raises(IndexError):
-		misc.item(e, -11)
+	assert misc.item(e, -11) is None
 
 	e = iter(range(10))
 	assert misc.item(e, -11, 42) == 42
@@ -132,51 +129,41 @@ def test_item():
 		misc.item(42, 42)
 
 	# Non-negative index
-	assert misc.item(iterable, 0), 17
-	assert misc.item(iterable, 2), 37
-	with pytest.raises(IndexError):
-		misc.item(iterable, 3)
-	assert misc.item(iterable, 3, 42), 42
-	assert misc.item(err(10), 9), 9
+	assert misc.item(iterable, 0) == 17
+	assert misc.item(iterable, 2) == 37
+	assert misc.item(iterable, 3) is None
+	assert misc.item(iterable, 3, 42) == 42
+	assert misc.item(err(10), 9) == 9
 	with pytest.raises(SyntaxError):
 		misc.item(err(10), 10)
 
 	# Negative index
-	assert misc.item(iterable, -1), 37
-	assert misc.item(iterable, -3), 17
-	with pytest.raises(IndexError):
-		misc.item(iterable, -4)
-	assert misc.item(iterable, -4, 42), 42
+	assert misc.item(iterable, -1) == 37
+	assert misc.item(iterable, -3) == 17
+	assert misc.item(iterable, -4) is None
+	assert misc.item(iterable, -4, 42) == 42
 	# iterator is always exhausted
 	with pytest.raises(SyntaxError):
 		misc.item(err(10), -1)
 
+	# Check index lists
+	assert misc.item(["foo", "bar"], (1, -1)) == "r"
+
 
 def test_first():
 	e = iter(range(10))
 	assert misc.first(e) == 0
 	assert misc.first(e) == 1
-
-	e = iter([])
-	with pytest.raises(IndexError):
-		misc.first(e)
-
-	e = iter([])
-	assert misc.first(e, 42) == 42
+	assert misc.first([]) is None
+	assert misc.first([], 42) == 42
 
 
 def test_last():
 	e = iter(range(10))
 	assert misc.last(e) == 9
-	with pytest.raises(IndexError):
-		misc.last(e)
-
-	e = iter([])
-	with pytest.raises(IndexError):
-		misc.last(e)
-
-	e = iter([])
-	assert misc.last(e, 42) == 42
+	assert misc.last(e) is None
+	assert misc.last([]) is None
+	assert misc.last([], 42) == 42
 
 
 def test_count():
@@ -241,17 +228,12 @@ def test_itersplitat():
 	assert tuple(misc.itersplitat("20090609172345", (-10, -8, -6, -4, -2))) == ("2009", "06", "09", "17", "23", "45")
 
 
-def test_gzip():
-	assert misc.gunzip(misc.gzip(b"gurk", 0)) == b"gurk"
-	assert misc.gunzip(misc.gzip(b"gurk", 9)) == b"gurk"
-
-
 def test_jsmin():
 	assert misc.jsmin("gurk \t = \t 42;") == "gurk=42;"
 
 
 def test_notimplemented():
-	class Bad(object):
+	class Bad:
 		@misc.notimplemented
 		def bad(self):
 			pass
@@ -271,7 +253,7 @@ def test_javaexpr():
 	assert str((1<<31)-1) == misc.javaexpr((1<<31)-1)
 	assert str(1<<31)+"L" == misc.javaexpr(1<<31)
 	assert str(-(1<<31)) == misc.javaexpr(-(1<<31))
-	assert str(-(1<<31)-1)+"L" == misc.javaexpr(-(1<<31)-1)
+	assert str(-(1<<31)-1) + "L" == misc.javaexpr(-(1<<31)-1)
 	assert str((1<<63)-1) + "L" == misc.javaexpr((1<<63)-1)
 	assert 'new java.math.BigInteger("{}")'.format(1<<64) == misc.javaexpr(1<<64)
 	# float
@@ -296,6 +278,10 @@ def test_javaexpr():
 	# dict
 	assert "com.livinglogic.utils.MapUtils.makeMap()" == misc.javaexpr({})
 	assert "com.livinglogic.utils.MapUtils.makeMap(1, 2)" == misc.javaexpr({1: 2})
+	# undefined
+	assert 'new com.livinglogic.ul4.UndefinedKey("foo")' == misc.javaexpr(ul4c.UndefinedKey("foo"))
+	assert 'new com.livinglogic.ul4.UndefinedVariable("foo")' == misc.javaexpr(ul4c.UndefinedVariable("foo"))
+	assert 'new com.livinglogic.ul4.UndefinedIndex(42)' == misc.javaexpr(ul4c.UndefinedIndex(42))
 
 
 def test_sysinfo():
@@ -308,11 +294,17 @@ def test_sysinfo():
 	misc.sysinfo.host_release
 	misc.sysinfo.host_version
 	misc.sysinfo.host_machine
+	misc.sysinfo.user_name
+	misc.sysinfo.user_uid
+	misc.sysinfo.user_gid
+	misc.sysinfo.user_gecos
+	misc.sysinfo.user_dir
+	misc.sysinfo.user_shell
 	misc.sysinfo.python_executable
 	misc.sysinfo.python_version
 	misc.sysinfo.pid
-	misc.sysinfo.scriptname
-	misc.sysinfo.shortscriptname
+	misc.sysinfo.script_name
+	misc.sysinfo.short_script_name
 
 
 def test_prettycsv():
@@ -320,3 +312,10 @@ def test_prettycsv():
 	assert "".join(misc.prettycsv([["a", "b "], ["abc", "def"]])) == "a     b\nabc   def\n"
 	assert "".join(misc.prettycsv([["a"], ["abc", "def"]])) == "a\nabc   def\n"
 	assert "".join(misc.prettycsv([["a", "b"], ["abc", "def"]], "..")) == "a  ..b\nabc..def\n"
+
+
+def test_timeout():
+	with pytest.raises(misc.Timeout):
+		with misc.timeout(3):
+			while True:
+				pass
